//*************************************************************************************
//	glcd.c
//
//	For a Crystalfontz CFAG320240CX Graphic LCD with Epson S1D13700F01 controller
//
// 	Screen 1 is text
//	Screen 2 is graphics
//	Screen 3 not used
//
//*************************************************************************************


#include "glcd.h"



//*************************************************************************************
//*************************************************************************************
int8 Hi ( int16 w )
	{
	return ( (w & 0xFF00) >> 8 );
	}




//*************************************************************************************
//*************************************************************************************
int8 Lo ( int16 w )
	{
	return ( w & 0x00FF );
	}




//*************************************************************************************
// Send instruction to the LCD
//*************************************************************************************
void lcd_CmdWrite (int8 Cmd)
	{
	// Put the command on the lines
	output_d ( Cmd );
   	delay_us ( 1 );

	output_high ( LCD_A0 );
   	output_low ( LCD_CS );
	delay_us ( 1 );

   	output_high ( LCD_E );
	delay_us ( 1 );
   	output_low ( LCD_E );

   	output_high ( LCD_CS );
	}



//*************************************************************************************
// Send data to the LCD
//*************************************************************************************
void lcd_DataWrite (int8 Data)
	{
	// Put the data on the lines
	output_d ( Data );
   	delay_us ( 1 );

	output_low ( LCD_A0 );
   	output_low ( LCD_CS );
	delay_us ( 1 );

   	output_high ( LCD_E );
	delay_us ( 1 );
   	output_low ( LCD_E );

   	output_high ( LCD_CS );
	}



//*************************************************************************************
// Read data from LCD
//*************************************************************************************
int8 lcd_ReadData ( void )
	{
	int8 rdata;

	// clear the data lines
	output_d ( 0x00 );

	// Setup the data lines as inputs
	set_tris_d ( 0xFF );

  	output_low ( LCD_E );
	output_high ( LCD_A0 );
  	output_low ( LCD_CS );
  	delay_us ( 1 );

  	output_high ( LCD_E );
  	delay_us ( 10 );
	rdata = input_d();
  	output_low ( LCD_E );

  	output_high ( LCD_CS );
	output_low ( LCD_A0 );
  	output_low ( LCD_RW );

	// Set the data lines back to being outputs
	set_tris_d ( 0x00 );

	// clear the data lines
	output_d ( 0x00 );


	return rdata;
	}



//*************************************************************************************
// Purpose: Get the current address of the cursor
// Outputs: A 16 bit integer containing the cursor address
//*************************************************************************************
int16 lcd_GetCursorAddress ( void )
{
	int8 addrl, addrh;

	lcd_CmdWrite(GLCD_CMD_GET_CSR_ADDR);
	addrl = LCD_ReadData();;
	addrh = LCD_ReadData();;

	return  ( addrl | ( addrh << 8 ) );
	}



//*************************************************************************************
// Purpose: Set the cursor address
// Inputs: 	A 16 bit integer containing the new cursor address
//*************************************************************************************
void lcd_SetCursorAddress ( int16 addr )
	{
	lcd_CmdWrite(GLCD_CMD_SET_CSR_ADDR);
	lcd_DataWrite( Lo(addr) );
	lcd_DataWrite( Hi(addr) );
	}



//*************************************************************************************
//*************************************************************************************
void lcd_ClearText ( void )
	{
	int16 i;

	lcd_CmdWrite(GLCD_CMD_SET_CSR_ADDR);
	lcd_DataWrite( Lo(TextPage) );
	lcd_DataWrite( Hi(TextPage) );

	lcd_CmdWrite(GLCD_CMD_DISPLAY_WRITE);
	output_low ( LCD_A0 );
   	output_low ( LCD_CS );
	output_d ( 0x20 );
	delay_us ( 1 );

	for ( i = 0; i != TextPageLength; i++ )
		{
		output_high ( LCD_E );
		delay_us ( 1 );
		output_low ( LCD_E );
		}

	output_high ( LCD_CS );

	lcd_SetCursorAddress ( TextPage );
	}


//*************************************************************************************
//*************************************************************************************
void lcd_ClearGraphics ( void )
	{
	int16 i;

	lcd_CmdWrite(GLCD_CMD_SET_CSR_ADDR);
	lcd_DataWrite( Lo(GraphicPage) );
	lcd_DataWrite( Hi(GraphicPage) );

	lcd_CmdWrite(GLCD_CMD_DISPLAY_WRITE);
	output_low ( LCD_A0 );
   	output_low ( LCD_CS );
	output_d ( 0x00 );
	delay_us ( 1 );

	for ( i = 0; i != GraphicPageLength; i++ )
		{
		output_high ( LCD_E );
		delay_us ( 1 );
		output_low ( LCD_E );
		}

	output_high ( LCD_CS );

	lcd_SetCursorAddress ( GraphicPage );
	}




//*************************************************************************************
// LCD Initialization
//
// Note: some command parameters are for 3 screens.  They work fine for the two
// 		 screen setup we are using.
//*************************************************************************************
void lcd_Init()
	{
	// Reset the display
	lcd_Reset();

	//System Setup
	lcd_CmdWrite( 0x40 );
	lcd_DataWrite( 0x30 );
	lcd_DataWrite( 0x87 );					//FX
	lcd_DataWrite( 0x07 );					//FY
	lcd_DataWrite( 0x27 ); 					//CR
	lcd_DataWrite( 0x42 ); 					//TC/R
	lcd_DataWrite( 0xEF ); 					//L/F
	lcd_DataWrite( 0x28 ); 					//APL
	lcd_DataWrite( 0x00 ); 					//APH


	// Scroll Setup
	lcd_CmdWrite( 0x44 );
	lcd_DataWrite( Lo(TextPage) );			// SAD1L
	lcd_DataWrite( Hi(TextPage) );			// SAD1H
	lcd_DataWrite( 0xEF );					// SL1
	lcd_DataWrite( Lo(GraphicPage) );		// SAD2L Lo(GLCD_GRAPHICS_ADDR)
	lcd_DataWrite( Hi(GraphicPage) );		// SAD2H Hi(GLCD_GRAPHICS_ADDR)
	lcd_DataWrite( 0xEF );					// SL2
	lcd_DataWrite( 0x00 ); 					// SAD3L
	lcd_DataWrite( 0x00 ); 					// SAD3H
	lcd_DataWrite( 0x00 ); 					// SAD4L
	lcd_DataWrite( 0x00 ); 					// SAD4H

	// Hor pixel scroll
	lcd_CmdWrite( 0x5A );
	lcd_DataWrite( 0x00 ); 					// scroll zero pixels

	// Overlay
	lcd_CmdWrite( 0x5B );
	lcd_DataWrite( 0x10 ); 					// 1-text, 2-graphic

	// Display Off
	lcd_CmdWrite( 0x58 );
	lcd_DataWrite( 0x55 ); 					// screen layers 1 and 2 enabled

	// Clear the screens
	lcd_ClearGraphics();
	lcd_ClearText();

	// Cursor Write
	//lcd_CmdWrite( 0x46 );					// Set cursor to the first screen block
	//lcd_DataWrite(0x00);				 	//
	//lcd_DataWrite(0x00);				 	//


	// Display On
	lcd_CmdWrite( 0x59 );
	lcd_DataWrite( 0x55 ); 					// screen layers 1 and 2 enabled

	// Cursor Direction Right
	lcd_CmdWrite( 0x4C );

	// Cursor Form
	lcd_CmdWrite( 0x5D );
	lcd_DataWrite( 0x00 );				 	// cursor width = 8
	lcd_DataWrite( 0x00 );				 	// cursor height = 8, block style


	}




//*************************************************************************************
// text memory starts at $0000. Organized in continous strip with wraparound.
//
// [0000][0001] ... [0039] ROW 0 (top of LCD)
// [0040][0041] ... [0079] ROW 1
// ...
// [1160[[1161] ... [1199] BOTTOM ROW
//
// [ ] values are decimal memory addresses
//
// Calculates mem address for ROW/COL and then sets cursor to that mem address
//*************************************************************************************
void lcd_GotoXY ( int8 x, int8 y )
	{
	int16 offset;

	if ( y > 29 )
		y = 29;

	if ( x > 39 )
		x = 39;

	offset = TextPage + (int16)y * 40;
	offset += x;
	lcd_SetCursorAddress ( offset );
	}




//*************************************************************************************
//*************************************************************************************
void lcd_Reset ( void )
	{
	output_low ( LCD_RST );

	output_d ( 0x00 );
	output_low ( LCD_RW );
   	output_high ( LCD_CS );
	output_low ( LCD_E );
	output_low ( LCD_A0 );

	delay_ms ( 10 );
	output_high ( LCD_RST );
	delay_ms ( 100 );
	}




//*************************************************************************************
//*************************************************************************************
void lcd_putc ( int8 ch )
	{
	lcd_CmdWrite ( GLCD_CMD_DISPLAY_WRITE );
    lcd_DataWrite ( ch );
	}



//*************************************************************************************
// Example for printing formatted output to the LCD
// 		char lcdstring[40];
// 		cnt = 15
// 		sprintf ( lcdstring, "Table %d is ready", cnt );
// 		LCD_puts ( lcdstring );
//*************************************************************************************
void lcd_puts ( char *s  )
	{
	char ch;

	lcd_CmdWrite ( GLCD_CMD_DISPLAY_WRITE );

	while ( ( ch = *s ) )
        {
        lcd_DataWrite ( ch );
        s++;
        }
	}



//*************************************************************************************
//*************************************************************************************
void lcd_fillScreenText(char c)
	{
	int16 i;

	lcd_SetCursorAddress ( TextPage );
	lcd_CmdWrite(GLCD_CMD_DISPLAY_WRITE);

	for(i = TextPage; i < TextPageLength; i++)
		{
		lcd_DataWrite(c);
		output_high ( LCD_E );
		output_low ( LCD_E );
		}

	}




//*************************************************************************************
//*************************************************************************************
void lcd_PutPixel ( int16 x, int16 y, int8 Pattern )
	{
	int16 MemAdr;
	int8 Offset;
	int8 Data;

	if ( x > 319 )
		x = 319;

	if ( y > 239 )
		y = 239;


	MemAdr = GraphicPage + y * 40 + x / 8;

	lcd_CmdWrite(GLCD_CMD_SET_CSR_ADDR);
	lcd_DataWrite( Lo(MemAdr) );
	lcd_DataWrite( Hi(MemAdr) );

	lcd_CmdWrite(GLCD_CMD_DISPLAY_READ);
	Data = LCD_ReadData();
	Pattern |= Data;
	//bit_set ( Data, 7 - x%8);

	lcd_CmdWrite(GLCD_CMD_SET_CSR_ADDR);
	lcd_DataWrite( Lo(MemAdr) );
	lcd_DataWrite( Hi(MemAdr) );

	lcd_CmdWrite(GLCD_CMD_DISPLAY_WRITE);
	lcd_DataWrite( Pattern );
	//lcd_DataWrite( Data );

	}



//*************************************************************************************
//*************************************************************************************
void lcd_VLine ( int16 x, int16 ystart, int16 yend, int8 Side )
	{
	int16 i, istart, iend; //used in indexing

	// need to determine direction to write data
	// this code sets istart to the lower of
	// ystart and yend. iend is then set to the
	// greater of ystart and yend
	if ( yend > ystart )
		{
		istart = ystart;
		iend = yend;
		}
	else
		{
		istart = yend;
		iend = ystart;
		}

	// to draw the vertical line, just write the pixels to the screen
	for ( i = istart;  i < iend; i++ )
		lcd_PutPixel( x, i, Side );

	}



//*************************************************************************************
//*************************************************************************************
void lcd_HLine ( int16 xstart, int16 y, int16 xend )
	{
	int16 i, istart, iend; //used in indexing

	// need to determine direction to write data
	// this code sets istart to the lower of
	// ystart and yend. iend is then set to the
	// greater of ystart and yend
	if ( xend > xstart )
		{
		istart = xstart;
		iend = xend;
		}
	else
		{
		istart = xend;
		iend = xstart;
		}

	// to draw the line, just write the pixels to the screen
	for ( i = istart;  i < iend; i++ )
		lcd_PutPixel( i, y, 0xFF );

	}



//*************************************************************************************
//*************************************************************************************
void lcd_Box ( int16 x1, int16 y1, int16 x2, int16 y2 )
	{
	lcd_VLine ( x1, y1, y2, LEFT );
	lcd_VLine ( x2, y1, y2, RIGHT );
	lcd_HLine ( x1, y1, x2 );
	lcd_HLine ( x1, y2, x2 );
	}
