//****************************************************************
//
//	For 320x240 Graphic LCD with Epson S1D13700F01 controller
//
//	Borrowed and modified from samples found in the 'net
//
//****************************************************************



#ifndef GLCD_H_
#define GLCD_H_

// Function Prototypes
int8  Hi ( int16 w );
int8  Lo ( int16 w );
void  lcd_DataWrite ( int8 );
void  lcd_CmdWrite ( int8 );
int8  lcd_ReadData ( void );
void  lcd_SetCursorAddress ( int16 addr );
int16 lcd_GetCursorAddress ( void );
void  lcd_ClearText ( void );
void  lcd_ClearGraphics ( void );
void  lcd_Init ( void );
void  lcd_GotoXY ( int8 x, int8 y );
void  lcd_Reset ( void );
void  lcd_putc ( int8 ch );
void  lcd_puts ( char *s  );
void  lcd_fillScreenText ( char c );
void  lcd_PutPixel ( int16 x, int16 y, int8 Pattern );
void  lcd_VLine ( int16 x, int16 ystart, int16 yend, int8 Side );
void  lcd_HLine ( int16 xstart, int16 y, int16 xend );
void  lcd_Box ( int16 x1, int16 y1, int16 x2, int16 y2 );



//****************************************************************
// The following defines setup the memory used by different regions
// Currenty one text area is defined at the beginning of memory
// and a graphics area follows immediately after
//****************************************************************
#define GLCD_TEXT_ADDR 			0x0000
#define GLCD_GRAPHICS_ADDR 		GLCD_WIDTH * GLCD_HEIGHT / 64
#define GLCD_GRAPHICS_ADDR_END 	GLCD_GRAPHICS_ADDR + (GLCD_WIDTH * GLCD_HEIGHT / 8)
//****************************************************************


// LCD Registers
#define GLCD_CMD_SYSTEM 		0x40 // General system settings
#define GLCD_CMD_SLEEP 			0x53 // Enter into standy mode
#define GLCD_CMD_DISP_OFF 		0x58 // Turn the display off
#define GLCD_CMD_DISP_ON 		0x59 // Turn the display on
#define GLCD_CMD_SCROLL 		0x44 // Setup text and graphics address regions
#define GLCD_CMD_CSR_FORM 		0x5D // Set cursor size
#define GLCD_CMD_CSRDIR_RIGHT 	0x4C // Cursor moves right after write to display memory
#define GLCD_CMD_CSRDIR_LEFT 	0x4D // Cursor moves left after write to display memory
#define GLCD_CMD_CSRDIR_UP 		0x4E // Cursor moves up after write to display memory
#define GLCD_CMD_CSRDIR_DN 		0x4F // Cursor moves down after write to display memory
#define GLCD_CMD_CGRAM_ADDR 	0x5C // Configure character generator RAM address
#define GLCD_CMD_HDOT_SCR 		0x5A // Set horizontal scroll rate
#define GLCD_CMD_OVERLAY 		0x5B // Configure how layers overlay
#define GLCD_CMD_SET_CSR_ADDR 	0x46 // Set the cursor address
#define GLCD_CMD_GET_CSR_ADDR 	0x47 // Read the cursor address
#define GLCD_CMD_DISPLAY_WRITE 	0x42 // Write to display memory
#define GLCD_CMD_DISPLAY_READ 	0x43 // Read from display memory


// General use definitions
#define ON 						1
#define OFF 					0
#define RIGHT					0x01
#define LEFT					0x80
#define NORMAL					0
#define INVERT					1
#define DATAOUT					1
#define DATAIN					0


// LCD configuration definitions
#define LARGE_LCD
#define GLCD_WIDTH 				320
#define GLCD_HEIGHT 			240
#define GLCD_CHAR_WIDTH 		8
#define GLCD_CHAR_HEIGHT 		8
#define TextPageLength			1200
#define GraphicPageLength		9600
#define TextPage				0
#define TextPageEnd				TextPage + TextPageLength
#define GraphicPage				TextPageEnd
#define GraphicPageEnd			GraphicPage + GraphicPageLength

#endif


