//**********************************************************************
// Code Sample.c
//
// For running the CFAG320240CX display with a PIC18F4520 at 20MHz
//
// See LCD_Sample_Schematic.pdf
//
// Compiled with CCS-C v3.236 under MPLAB
//
// This is an excerpt from a running application.
//
// Display Characteristics
// LED backlight with 10 levels (minimize writes to the EEPROM}
// Default 6800 mode
// Screen 1 is text
// Screen 2 is graphics
//
//
// NOTE:
//	There is something weird about the way I am doing the PutPixel routines
//	in glcd.c.  It is causing me to have to specify the beginning or end
//	of an eight bit block when I draw a vertical line.
//  This is handled by the Side parameter to lcd_VLine.  Either RIGHT or
//	LEFT is specified.  It works for what I needed right now.
//
//	If somebody can explain to me what I'm doing wrong I'd appreciate it.
//**********************************************************************

#include <18F4520.h>
#use delay(clock=20000000)



//**********************************************************************
// Function Prototypes for Main.c
void SystemInit ( void );
void main();
//**********************************************************************


#include "glcd.c"


// LCD data pins are on RD0 - RD7
#define Backlight	PIN_C1		// This pin is used as PWM2 (CCP2)
#define LCD_RW		PIN_A4 		// Read/Write, 1 = Read, 0 = Write
#define LCD_E		PIN_A5 		// Enable, 1 = Enable for Read or Write, 0 = No operation
#define LCD_A0		PIN_B4
#define LCD_CS		PIN_B5 		// Chip Select, 0 = Active
#define LCD_RST		PIN_C7 		// Reset, 0 = Active



//**********************************************************************
// Globals
char 	lcdstring[80];
int8	BacklightStep;
//**********************************************************************



//**********************************************************************
// For storing and handling the backlight level
#define	BacklightFactor		52
#define	BacklightLoc		0xF00000    //1 bytes
//**********************************************************************




//**********************************************************************
// SystemInit
//**********************************************************************
void SystemInit ( void )
    {
	#use fast_io (A)
	#use fast_io (B)
	#use fast_io (C)
	#use fast_io (D)
	#use fast_io (E)

	set_tris_a ( 0b00000000 );
    set_tris_b ( 0b00000000 );
    set_tris_c ( 0b00000000 );
	set_tris_d ( 0b00000000 );
    set_tris_e ( 0b00000000 );


    // Get the backlight level stored in eeprom
    // If this is the first time the value in EEPROM might be undefined
    BacklightStep = read_eeprom ( BacklightLoc );
    if ( BacklightStep > 10 )
    	BacklightStep = 0;

    // Setup the PWM to control the LCD backlighting
    // Zero to 520 is the range from no backlight to full backlight
    // We will use 10 steps to accomplish this.
    // Mostly this is to minimize the number of EEPROM writes so
    // we don't wear it out too fast.
   	setup_ccp2 ( CCP_PWM );
   	setup_timer_2 ( T2_DIV_BY_16, 127, 1 );
   	set_pwm2_duty( (int16)BacklightStep * (int16)BacklightFactor );

    // Initialize the display.  Function in glcd.c
    lcd_Init ();
	}



//**********************************************************************
// main
//**********************************************************************
void main()
	{
	SystemInit();

	//lcd_VLine ( 70, 0, 239 );
	//lcd_VLine ( 210, 0, 239 );
	//lcd_HLine ( 0, 45, 319 );
	//lcd_HLine ( 0, 58, 319 );

	lcd_Box ( 100, 45, 210, 58 );
	sprintf ( lcdstring, "1234567890123456789012345678901234567890" );
	lcd_GotoXY ( 0, 29 );
	lcd_puts ( lcdstring );

	while ( 1 )
		{
		if ( input( BL_UP_Btn )
			{
			// Wait until they take their finger off the button
			while ( input ( BL_UP_Btn ) )
				delay_ms ( 100 );

			if ( BacklightStep >= 1 )
				{
				--BacklightStep;
				set_pwm2_duty( (int16)BacklightStep * (int16)BacklightFactor );
				write_eeprom ( BacklightLoc, BacklightStep );
				}
			}

		if ( input( BL_DN_Btn )
			{
			// Wait until they take their finger off the button
			while ( input ( BL_DN_Btn ) )
				delay_ms ( 100 );

			if ( BacklightStep <= 9 )
				{
				++BacklightStep;
				set_pwm2_duty( (int16)BacklightStep * BacklightFactor );
				write_eeprom ( BacklightLoc, BacklightStep );
				}
			}
		}
	}


